/**
 * @file liboattnusb.h
 *
 * @brief OASIS USB Attenuator driver library interface
 *
 * This file describes the external interface to the OASIS USB Attenuator
 * driver and provides the definitios for proper communication
 * with the device
 *
 * Copyright (c) 2018 CERN
 * @author Tomasz Włostowski <tomasz.wlostowski@cern.ch>
 *
 * @section license_sec License
 * Released under the GPL v2. (and only v2, not any later version)
 */


#ifndef __LIBOATTNUSB_H
#define __LIBOATTNUSB_H

#ifdef __cplusplus
extern "C" {
#endif

/** \mainpage OASIS USB attenuator library
*
* \section intro_sec Introduction
*
* To be able to use the library API, you just need to include the
* .liboattnusb.h header file. Linkage requires the flag
* `-loattnusb` and, in addition, `-lftdi1 -ludev`, as these are
* used by the basic library. To wit:
*
*		gcc -o test test.c \
*			-L /acc/local/L867/drv/oattn/1.0/lib \
*			-I/acc/local/L867/drv/oattn/1.0/include \
*			-loattnusb -lftdi1 -ludev
*
* In case the static version of the library is preferred, a more
* contorted incantation than  the above may be required to
* the static library version in a foolproof manner:
*
*		gcc -o test test.c \
*			-L /acc/local/L867/drv/oattn/1.0/lib \
*			-I/acc/local/L867/drv/oattn/1.0/include \
*			-lftdi1 -ludev \
*			-Wl,--push-state,-static \
*			-loattnusb \
*			-Wl,--pop-state
*
* \section api_sec API
*
* The library API is documented in liboattnusb.h
*/

/** 
 * @enum oattnusb_value_enum
 * @brief Valid relay values
 *
 */
enum oattnusb_value_enum {
	ATT_ERR	=	-1,	/**< Error */
	ATT_MINUS_40dB = 0x0,	/**< -40dB attn (power-on default) */
	ATT_MINUS_20dB = 0x01,	/**< -20dB attn */
	ATT_ZERO_dB = 	 0x03,	/**<   0dB attn */
};

/**
 * @brief Open a msatt device by LUN (not implemented)
 *
 * A board is selected by lun, and a handle is returned for further
 * reference to it.
 *
 * @param lun - logical unit number of USB attenuator
 *
 * @return >0 - on success, device file descriptor number
 */
int oattnusb_open(int lun);

/**
 * @brief Open a msatt device
 *
 * Open OASIS USB attenuator with given serial number.
 *
 * @param serial - serial number of USB attenuator
 *
 * @return >0 - on success, device file descriptor number
 */
int oattnusb_open_by_serial(const char *serial);

/**
 * @brief close a channel handle
 *
 * @param fd  Handle to close
 */
int oattnusb_close(int fd);

/**
 * @brief Get attenuation value for a given channel
 * 
 * Each channel can be in one of these states:
 *
 *            - attenuation -40dB (power-on default) (Relay value = 0x0)
 *            - attenuation -20dB (Relay value = 0x1)
 *            - attenuation   0dB (Relay value = 0x3)
 *
 * @param fd 		Device handle identifying board
 * @param channel 	Channel [1,2,3,4] on the board to read from
 * 
 * @return ATT_ERR on failure, another oattnusb_value_enum value on success
 */
enum oattnusb_value_enum oattnusb_get_relay(int fd, int channel);

/**
 * @brief Set attenuation value for a given channel
 * 
 * @param fd		Device handle identifying board
 * @param channel 	Channel on the board [1,2,3,4] 
 * @param val 		Valid oattnusb_value_enum relay value
 * 
 * @return Upon successful completion, returns a value other than -1. On failure it returns -1
 */
int oattnusb_set_relay(int fd, int channel, enum oattnusb_value_enum val);

/**
 * @brief Gets the number of channels (2 or 4)
 * 
 * @param fd		Device handle identifying board
 * 
 * @return -1 on fail, 2 for MSATTN2 and 4 for MSATTN4
 */
int oattnusb_get_nchannels(int fd);

/* libmsatt version string */
extern const char * const liboattnusb_version_s;
#ifdef __cplusplus
}
#endif

#endif /* __LIBOATTNUSB_H */
